#include <stdint.h>

#include <linux.h>
#include <sdram.h>
#include <serial.h>
#include <utils.h>

typedef void (*kernel_t) (int, int, void*);

static const uint32_t machine_id = 2214;

static int
cmdcpy(uint32_t *dst, const char *src)
{
	char *dstc = (char*) dst;
	int len = 0;

	while (*src) {
		*dstc++ = *src++;
		++len;
	}

	while (len & 3) {
		*dstc++ = ' ';
		++len;
	}

	return (len >> 2);
}

void linux_go(const char *cmdline)
{
	kernel_t kernel = (kernel_t) LINUX_BASE_ADDRESS;
	uint32_t *tags = (uint32_t*) LINUX_TAGS_ADDRESS;
	uint32_t *sz;
	int len;

	/* atag_core */
	*tags++ = 5;
	*tags++ = 0x54410001;
	*tags++ = 1;
	*tags++ = 4096;
	*tags++ = 0;

	/* atag_mem */
	*tags++ = 4;
	*tags++ = 0x54410002;
	*tags++ = SDRAM_BASE_ADDRESS;
	*tags++ = SDRAM_SIZE;

	/* atag_cmdline */
	sz = tags++;
	*tags++ = 0x54410009;
	len = cmdcpy(tags, cmdline);
	*sz = 2 + len;
	tags += len;

	/* atag_none */
	*tags++ = 0;
	*tags++ = 0;

	serial_printk("Jump to the kernel image\n\n");

	kernel(0, machine_id, (void*) LINUX_TAGS_ADDRESS);

	for (;;) ;
}

