#include <stdint.h>
#include <stdarg.h>

#include <serial.h>
#include <AT91RM9200.h>
#include <config.h>
#include <utils.h>

#define DCHAR(oct)		(0x30+(oct))
#define HCHAR(oct)		(((oct)<10)?(0x30+(oct)):(0x41+(oct)-10))

void serial_init()
{
	uint32_t tmp32;
	unsigned int cd;

	tmp32 = (AT91C_PA30_DRXD | AT91C_PA31_DTXD);
	AT91C_BASE_PIOA->PIO_ASR = tmp32;
	AT91C_BASE_PIOA->PIO_PDR = tmp32;

	AT91C_BASE_PMC->PMC_PCER = (1 << AT91C_ID_SYS);

	__u32divmod((clk_mck << 1), (115200 << 4), &cd, NULL);
	cd = (cd + 1) >> 1;

	AT91C_BASE_DBGU->DBGU_IDR = 0xffffffff;
	AT91C_BASE_DBGU->DBGU_CR = (AT91C_US_RSTRX | AT91C_US_RSTTX | AT91C_US_RXDIS | AT91C_US_TXDIS);
	AT91C_BASE_DBGU->DBGU_MR = (AT91C_US_PAR_NONE | AT91C_US_CHMODE_NORMAL);
	AT91C_BASE_DBGU->DBGU_BRGR = cd;
	AT91C_BASE_DBGU->DBGU_CR = (AT91C_US_TXEN | AT91C_US_RXEN);
}

/* FIXME: Use PDC & FIFOs here
 * */
char serial_getc()
{
	while (!(AT91C_BASE_DBGU->DBGU_CSR & AT91C_US_RXRDY));
	return AT91C_BASE_DBGU->DBGU_RHR;
}

void serial_putc(char c)
{
	while (!(AT91C_BASE_DBGU->DBGU_CSR & AT91C_US_TXRDY));
	AT91C_BASE_DBGU->DBGU_THR = c;
}

static void
serial_printk_hex(uint32_t val, int h)
{
	uint32_t mask = 0xf0000000 >> (32 - 4*h);

	while (h != 0) {
		serial_putc(HCHAR((val & mask) >> (4*h - 4)));
		mask = mask >> 4;
		--h;
	}
}

static void
serial_printk_dec(uint32_t val)
{
	char buf[16];
	char *ptr;
	uint32_t nval;
	uint32_t n, num;

	n = sizeof(buf) - 1;
	buf[n] = 0x0;

	do {
		__u32divmod(val, 10, &nval, &num);
		buf[--n] = DCHAR(num);
		val = nval;
	}
	while (val);

	ptr = &buf[n];

	while (*ptr) {
		serial_putc(*ptr++);
	}
}

void serial_printk(const char *str, ...)
{
	va_list ap;
	const char *sarg;

	va_start(ap, str);
	while (*str) {
		if (*str == '%') {
			++str;
			switch (*str) {
				case 'x':
					serial_printk_hex(va_arg(ap, uint32_t), 8);
					break;
				case 'h':
					serial_printk_hex(va_arg(ap, uint32_t), 4);
					break;
				case 'b':
					serial_printk_hex(va_arg(ap, uint32_t), 2);
					break;
				case 'i':
					serial_printk_dec(va_arg(ap, uint32_t));
					break;
				case 's':
					sarg = va_arg(ap, const char*);
					while (*sarg)
						serial_putc(*sarg++);
					break;
			}
		}
		else
			serial_putc(*str);
		++str;
	}
	va_end(ap);
}

