#include <stdint.h>

#include <spi.h>
#include <AT91RM9200.h>
#include <config.h>

static int csdec[] = {
	0x000e0000,
	0x000d0000,
	0x000b0000,
	0x00070000,
};

void spi_init()
{
	uint32_t tmp32;

	tmp32 = (	AT91C_PA0_MISO |
			AT91C_PA1_MOSI |
			AT91C_PA2_SPCK |
			AT91C_PA3_NPCS0 |
			AT91C_PA4_NPCS1 |
			AT91C_PA5_NPCS2 |
			AT91C_PA6_NPCS3 );

	AT91C_BASE_PIOA->PIO_ASR = tmp32;
	AT91C_BASE_PIOA->PIO_PDR = tmp32;

	AT91C_BASE_PMC->PMC_PCER = (1 << AT91C_ID_SPI);

	AT91C_BASE_SPI->SPI_IDR = 0xffffffff;
	AT91C_BASE_SPI->SPI_CR = AT91C_SPI_SWRST;
	AT91C_BASE_SPI->SPI_MR = (AT91C_SPI_MSTR |
			AT91C_SPI_MODFDIS | AT91C_SPI_PCS | (0x1f << 24));
	AT91C_BASE_SPI->SPI_CR = AT91C_SPI_SPIEN;
}

void spi_mode(int cs, int pol, int bits, int baud, int dely)
{
	AT91C_BASE_SPI->SPI_CSR[cs] = (pol | ((bits - 8) << 4) | (baud << 8) |
			((dely) << 24) | ((dely >> 1) << 16));
}

void spi_select(int cs)
{
	uint32_t tmp32 = AT91C_BASE_SPI->SPI_MR;
	AT91C_BASE_SPI->SPI_MR = (tmp32 & ~(0x000f0000)) | csdec[cs];
}

void spi_transfer(struct spi_tblk *blk, int n)
{
	AT91C_BASE_SPI->SPI_PTCR = AT91C_PDC_TXTDIS | AT91C_PDC_RXTDIS;

	AT91C_BASE_SPI->SPI_TPR = (uint32_t) blk[0].tx;
	AT91C_BASE_SPI->SPI_TCR = blk[0].len;
	AT91C_BASE_SPI->SPI_RPR = (uint32_t) blk[0].rx;
	AT91C_BASE_SPI->SPI_RCR = blk[0].len;

	if (n > 1) {
		AT91C_BASE_SPI->SPI_TNPR = (uint32_t) blk[1].tx;
		AT91C_BASE_SPI->SPI_TNCR = blk[1].len;
		AT91C_BASE_SPI->SPI_RNPR = (uint32_t) blk[1].rx;
		AT91C_BASE_SPI->SPI_RNCR = blk[1].len;
	}

	AT91C_BASE_SPI->SPI_PTCR = AT91C_PDC_TXTEN | AT91C_PDC_RXTEN;
	while (!(AT91C_BASE_SPI->SPI_SR & AT91C_SPI_RXBUFF)) ;

	AT91C_BASE_SPI->SPI_PTCR = AT91C_PDC_TXTDIS | AT91C_PDC_RXTDIS;
}

