#include <stdint.h>

#include <at45.h>
#include <config.h>
#include <ext2.h>
#include <linux.h>
#include <mci_sd.h>
#include <sdram.h>
#include <serial.h>
#include <utils.h>
#include <xmod.h>

void zboot_start() __attribute__ ((__noreturn__));
void zboot_abort() __attribute__ ((__noreturn__));

void zboot_abort()
{
	serial_printk("\nAbort\n");
	for (;;) ;
}

static const char *linux_cmdline;

static int
zboot_exec(char *cmd, char *args)
{
	if (strequ(cmd, "clock")) {
		char *clk = args;

		while (*args != ' ') ++args;
		*args++ = 0;
		while (*args == ' ') ++args;
		zboot_reinit_clock(strtou32(clk) * MHZ,
				strtou32(args));
		return 0;
	}
	else if (strequ(cmd, "cmdline")) {
		linux_cmdline = args;
		return 0;
	}
	else if (strequ(cmd, "kernel")) {
		ext2_read(args, (void*) LINUX_BASE_ADDRESS);
		linux_go(linux_cmdline);
		return 0;
	}
	else if (strequ(cmd, "flash")) {
		uint32_t len, pad;
		void *ptr;

		serial_printk("Flash updating\n");

		ptr = (void*) (LINUX_BASE_ADDRESS + (1 << 20));
		len = ext2_read(args, ptr);
		__u32divmod(len, AT45_PAGE_SIZE, NULL, &pad);
		len += (pad) ? (AT45_PAGE_SIZE - pad) : 0;

		if (at45_write(0, ptr, len) < 0)
			serial_printk("Flash bad write\n");

		return 0;
	}

	return -1;
}

static int
zboot_runseq(char *seq)
{
	char *cmd, *arg;

	do {
		while (*seq == ' ') ++seq;

		cmd = seq;
		while (*seq != ' ') ++seq;

		*seq++ = 0;
		while (*seq == ' ') ++seq;

		arg = seq;
		while (*seq != '\n') ++seq;

		*seq++ = 0;

		if (zboot_exec(cmd, arg) < 0)
			return -1;
	}
	while (1);

	return 0;
}

void zboot_start()
{
	char *str;
	int len;

	serial_printk("\nZBoot (Linux loader for AT91)\n");

	sdram_info();

	if (at45_init(0) < 0) {
		zboot_abort();
	}

	if (mci_sd_init() < 0) {
		zboot_abort();
	}

	if (ext2_mount(mci_sd_read, 0) < 0) {
		zboot_abort();
	}

	str = (void*) SDRAM_BASE_ADDRESS;
	len = ext2_read("zboot.seq", str);

	if (len > 0) {
		strcopy(str + len, "\nx x\n");

		if (zboot_runseq(str) < 0) {
			zboot_abort();
		}
	}

	/*xmod_recv((void*) LINUX_BASE_ADDRESS);
	linux_go("mem=64M root=/dev/sda2 rootwait ro");*/

	for (;;) ;
}

