#include <stdint.h>

#include <AT91RM9200.h>
#include <config.h>
#include <sdram.h>
#include <serial.h>
#include <spi.h>
#include <utils.h>

#define CLOCK_DIVIDER		0x00000004
#define CLOCK_PLLAR_VAL		(0x2000bf00 | CLOCK_DIVIDER)
#define CLOCK_PLLBR_VAL		(0x10173f00 | CLOCK_DIVIDER)

unsigned int clk_mck;

static void
wait_lock(int bit)
{
	int i = 0;
	while (!(AT91C_BASE_PMC->PMC_SR & bit) && (i++ < 128))
		nops(16);
}

static void
z_init_clock(unsigned int plla_clk, unsigned int mdiv)
{
	int regval;
	int pck = plla_clk;

	AT91C_BASE_PMC->PMC_IDR = 0xffffffff;

	regval = AT91C_CKGR_MOSCEN | (0xff << 8);
	if (AT91C_BASE_CKGR->CKGR_MOR != regval) {
		AT91C_BASE_CKGR->CKGR_MOR = regval;
		wait_lock(AT91C_PMC_MOSCS);
	}

	__u32divmod(plla_clk, MHZ, &plla_clk, NULL);
	__u32divmod(plla_clk, CLOCK_DIVIDER, &plla_clk, NULL);

	regval = CLOCK_PLLAR_VAL |
		((plla_clk - 1) << 16);
	if (AT91C_BASE_CKGR->CKGR_PLLAR != regval) {
		AT91C_BASE_CKGR->CKGR_PLLAR = regval;
		wait_lock(AT91C_PMC_LOCKA);
	}

	regval = CLOCK_PLLBR_VAL;
	if (AT91C_BASE_CKGR->CKGR_PLLBR != regval) {
		AT91C_BASE_CKGR->CKGR_PLLBR = regval;
		wait_lock(AT91C_PMC_LOCKB);
	}

	regval = ((mdiv - 1) << 8) | AT91C_PMC_CSS_PLLA_CLK;
	if (AT91C_BASE_PMC->PMC_MCKR != regval) {
		AT91C_BASE_PMC->PMC_MCKR = regval;
		wait_lock(AT91C_PMC_MCKRDY);
	}

	if (AT91C_BASE_PMC->PMC_SCER != AT91C_PMC_PCK) {
		AT91C_BASE_PMC->PMC_SCER = AT91C_PMC_PCK;
	}

	__u32divmod(pck, mdiv, &clk_mck, NULL);
}

void zboot_init()
{
	z_init_clock(180 * MHZ, 3);

	sdram_init();
	serial_init();
	spi_init();

	nops(1000);
}

void zboot_reinit_clock(unsigned int pck, unsigned int div)
{
	serial_printk("Reset Clock to %i Hz [Divider = %i]\n", pck, div);

	z_init_clock(pck, div);

	serial_init();

	nops(1000);
}

