/* Terminal program
 *
 * Usage: term [FILE]
 *
 * Transfer FILE with Xmodem protocol, and terminal session, 
 * for exit press Ctrl-X .
 *
 * */

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <termios.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>
#include <string.h>
#include <errno.h>
#include <sys/select.h>

#include "tty.h"
#include "xmod.h"

#define TBUF_LENGTH		32

static struct termios set_sv;
static int run = 1;

static void
term_restote()
{
	if (tcsetattr(0, TCSANOW, &set_sv) < 0) {
		fprintf(stderr, "tcsetattr failed: %s \n", strerror(errno));
		exit(EXIT_FAILURE);
	}
}

static void
term_handler(int signo)
{
	exit(EXIT_FAILURE);
}

int main(int argc, char *argv[])
{
	struct sigaction act;
	act.sa_handler = term_handler;
	act.sa_flags = 0;
	sigemptyset(&act.sa_mask);
	sigaction(SIGTERM, &act, 0);
	sigaction(SIGINT, &act, 0);

	struct termios set;
	char tbuf[TBUF_LENGTH];
	int dev = tty_open("/dev/ttyS0");
	fd_set fds;
	int nfds;
	int ret, len, i;

	if (argc > 1) {
		fprintf(stderr, "xmodem: uploading \n");
		if (xmod_load(dev, argv[1]) < 0) {
			exit(EXIT_FAILURE);
		}
		fprintf(stderr, "xmodem: done");
	}

	printf("\n");
	fflush(stdout);

	if (tcgetattr(0, &set) < 0) {
		fprintf(stderr, "tcgetattr failed: %s \n", strerror(errno));
		exit(EXIT_FAILURE);
	}

	memcpy(&set_sv, &set, sizeof(struct termios));
	set.c_lflag &= ~(ISIG | ICANON | ECHO | ECHOE);

	if (tcsetattr(0, TCSANOW, &set) < 0) {
		fprintf(stderr, "tcsetattr failed: %s \n", strerror(errno));
		exit(EXIT_FAILURE);
	}

	atexit(term_restote);

	while (run) {
		FD_ZERO(&fds);
		FD_SET(0, &fds);
		FD_SET(dev, &fds);

		nfds = dev;

		ret = select(nfds + 1, &fds, NULL, NULL, NULL);

		if (ret) {
			if (FD_ISSET(0, &fds)) {
				len = read(0, tbuf, TBUF_LENGTH);
				if (len > 0) {

					for (i = 0; i < len; ++i) {
						if (tbuf[i] == 0x18)
							run = 0;
					}

					if (run) {
						write(dev, tbuf, len);
					}
				}
			}

			if (FD_ISSET(dev, &fds)) {
				len = read(dev, tbuf, TBUF_LENGTH);
				if (len > 0) {
					write(1, tbuf, len);
				}
			}
		}
	}

	tty_close(dev);

	return 0;
}

