#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <termios.h>

#include "tty.h"

int tty_open(const char *name)
{
	struct termios set;

	int sd = open(name, O_RDWR | O_NOCTTY);
	if (sd < 0) {
		fprintf(stderr, "tty open failed: %s \n", strerror(errno));
		exit(EXIT_FAILURE);
	}

	if (tcgetattr(sd, &set) < 0) {
		fprintf(stderr, "tcgetattr failed: %s \n", strerror(errno));
		exit(EXIT_FAILURE);
	}

	cfsetispeed(&set, B115200);
	cfsetospeed(&set, B115200);

	set.c_iflag &= ~( IGNBRK | BRKINT | PARMRK | ISTRIP
			| INLCR | IGNCR | ICRNL | IXON );
	set.c_oflag &= ~OPOST;
	set.c_lflag &= ~( ICANON | ECHO | ECHOE | ISIG );
	set.c_cflag &= ~( CSTOPB | PARENB | CSIZE );
	set.c_cflag |= ( CS8 | CLOCAL | CREAD );

	if (tcsetattr(sd, TCSANOW, &set) < 0) {
		fprintf(stderr, "tcsetattr failed: %s \n", strerror(errno));
		exit(EXIT_FAILURE);
	}

	return sd;
}

void tty_close(int sd)
{
	if (close(sd) < 0) {
		fprintf(stderr, "tty close failed: %s \n", strerror(errno));
		exit(EXIT_FAILURE);
	}
}

