#include <stdint.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <sys/stat.h>

#include "xmod.h"

uint16_t crc16(uint8_t *ptr, int count)
{
	uint16_t crc, i;
	crc = 0x0000;

	while (count--) {
		crc ^= (uint16_t) *ptr++ << 8;
		for(i = 0; i < 8; ++i)
			crc = crc & 0x8000 ? (crc << 1) ^ 0x1021 : crc << 1;
	}

	return crc;
}

int xmod_load(int fd, const char *name)
{
	uint8_t c;
	uint8_t block[128];
	uint16_t crc;
	struct stat fst;
	int len, rlen;
	int dlen, slen;
	int blk, blk_n;
	int err = 0;

	int file = open(name, O_RDONLY);

	if (file < 0) {
		fprintf(stderr, "file open failed: %s \n", strerror(errno));
		return -1;
	}

	if (fstat(file, &fst) < 0) {
		fprintf(stderr, "file fstat failed: %s \n", strerror(errno));
		return -1;
	}

	blk_n = fst.st_size / 128;
	slen = fst.st_size % 128;
	++blk_n;

	for (blk = 1; blk <= blk_n; ++blk) {

		dlen = (blk == blk_n) ? slen : 128;

		len = 0;
		do {
			rlen = read(file, block + len, dlen - len);
			if (rlen < 0) {
				fprintf(stderr, "\nfile read failed: %s \n", strerror(errno));
				return -1;
			}
			else {
				len += rlen;
			}
		} while (len < dlen);

		err = 0;
		crc = crc16(block, 128);

		do {
			c = XM_SOH;
			write(fd, &c, 1);
			c = (uint8_t) blk;
			write(fd, &c, 1);
			c = ~((uint8_t) blk);
			write(fd, &c, 1);

			write(fd, block, 128);
			c = crc >> 8;
			write(fd, &c, 1);
			c = crc;
			write(fd, &c, 1);

			read(fd, &c, 1);

			if (c != XM_ACK)
				++err;
			else
				break;

			if (err > 4) {
				fprintf(stderr, "\nxmodem failed: %s \n", "more than 4 times send failed");
				return -1;
			}

		} while (1);

		fprintf(stderr, "#");
		fflush(stderr);
	}

	fprintf(stderr, "\n");

	c = XM_EOT;
	write(fd, &c, 1);

	read(fd, &c, 1);
	if (c != XM_ACK) {
		fprintf(stderr, "xmodem failed: %s \n", "expected ACK charaster");
		return -1;
	}

	if (close(file) < 0) {
		fprintf(stderr, "file close failed: %s \n", strerror(errno));
		return -1;
	}

	return 0;
}

